/*
 * esp.c  -  process ESP host adapters.
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         jdd@cdf.toronto.edu
 * 
 */

#ifndef lint
static char rcsid[] = "$Id: esp.c,v 1.14 1999/04/22 17:40:16 jdd Exp $";
#endif

#ifdef ESP_SUPPORT

#include "uscsi.h"
#include "format.h"
#include <stdio.h>

#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <nlist.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include "utils.h"
#include "common.h"
#include "param.h"

/* include OS-dependent stuff */
#ifdef SYSV
#include "sunos5.h"
#include <sys/scsi/adapters/espvar.h>
#else 
#ifdef SUNOS4_1_0
/* no espvar.h in SunOS 4.1 */
#include <scsi/adapters/espreg.h>
#else /* !SUNOS4_1_0 */
#include <scsi/adapters/espvar.h>
#endif /* !SUNOS4_1_0 */
#include "sunos4.h"
#endif /* !SYSV */

/*  
 * SunOS 5.6 got rid of e_targets, so use e_target_scsi_options_defined
 * instead.  We see if we're running 5.6 by checking for 
 * ESP_OPT_SLOW_FIFO_FLUSH which first showed up in 5.6.
 */
#ifdef ESP_OPT_SLOW_FIFO_FLUSH
#define e_targets e_target_scsi_options_defined
#ifndef ESP_TRUSTING
#define ESP_TRUSTING /* e_target_scsi_options_defined is often zero */
#endif /* ESP_TRUSTING */
#endif /* ESP_OPT_SLOW_FIFO_FLUSH */

static char *espType(typecode)
unsigned int typecode;
{
	/* see /usr/include/[sys/]scsi/adapters/espreg.h */
	switch(typecode){
	case 0:		return("ESP100/NCR53C90");
	case 1: 	return("ESP100A/NCR53C90A");
	case 2:		return("ESP236");
	case 3:		return("FAS100/100A");
	case 4:		return("FAS236");
	case 5:		return("FAST");
	default:	return("unknown");
	}
}

static u_int esp_get_speed(esp, i)
struct esp *esp;
int i; /* target index */
{
	u_int cycletime, speed;

	/*
	 * Synch cycle time in milliseconds is the
	 * number of ticks in the period divided
	 * by the tick rate. e_clock_conv is the
	 * esp's tick rate in units of 5MHz.
	 */
	cycletime = esp->e_clock_conv ? (esp->e_period[i] *
			(1000/(int)(5*esp->e_clock_conv))) : 0;

	/*
	 * Compute speed in thousanths of MBs/sec.
	 */
	speed = cycletime?((((1000*1000*1000)/cycletime)+999)/1000):0;

	return(speed);
}

static void esp_controller_message(cnum, esp)
int cnum;
struct esp *esp;
{
	printf("esp%d:", cnum);
	printf(" type %d (%s)", (u_int)esp->e_type,
		espType(esp->e_type));
	printf(" speed %dMHz", (u_int)esp->e_clock_conv*5);
	printf("\n");
}

static void esp_synch_message(esp, i, speed)
struct esp *esp;
int i;
u_int speed;
{
#ifndef ESP_TRUSTING
	if(esp->e_weak & ~(esp->e_targets)) {
		/* don't believe esp synch info if e_weak mentions 
		 * unprobed targets; something has gone awry. Some versions of 
		 * the 5.x ct driver for the Central Data STS do this.
		 */
		if(0==esp->e_type){
			/* Since this host adapter uses an NCR 53C90, not a 
			 * 53C90A or later, e_weak is bogus probably because 
			 * synch scsi is turned off in the kernel (53C90s are
			 * overly susceptible to noise when doing synchronous 
			 * transfers). See esp_ss1_esp0sync.
			 */
			printf("Asynchronous");
		} else {
			printf("Unknown");
		}
	} else 
#endif /* ESP_TRUSTING */
	if(speed && (esp->e_sync_known & (1<<i))){
		printf("Synchronous(%d.%.3dMB/sec)", speed/1000, speed%1000);
	} else {
		printf("Asynchronous");
	}
#ifndef ESP_TRUSTING
	if(esp->e_weak & ~(esp->e_targets)) {
		/* again, don't believe e_weak if mentions unprobed targets */
		printf(" Unknown");
	} else 
#endif /* ESP_TRUSTING */
	if(esp->e_weak & (1<<i)){
		printf(" Noisy");
	} else {
		printf(" Clean");
	}
#ifndef ESP_TRUSTING
	if(esp->e_nodisc & ~(esp->e_targets)) {
		/* don't believe e_nodisc if it mentions unprobed targets */
		printf(" Unknown");
	} else 
#endif /* ESP_TRUSTING */
	if(esp->e_nodisc & (1<<i)){
		printf(" CannotReconnect");
	} else {
		printf(" CanReconnect");
	}
}

int process_esp()
{
	u_long addr;
	struct esp esp[1];

#ifdef OLD_SUNOS
	struct esp *esp_softc, *esp_p;
#endif /* OLD_SUNOS */

	int noisycount=0; /* number of noisy devices */
	u_int j, nesp;

        if(-1==kvm(nl[N_ESP_SOFTC].n_value, addr)){
		return(0); /* esp driver not loaded */
        }
        if((u_long)NULL==addr) return(0); /* no esp's */

        if(-1==kvm(addr, esp[0])){
                Error("kvm read: initial esp");
        }

#ifdef OLD_SUNOS
	esp_softc=esp_p=(struct esp *)addr;
#endif /* OLD_SUNOS */

	nesp=get_nesp();
	if(0==nesp) return(0);

	for(j=0;j<nesp;j++){
		struct dev_info di, *next;
		char *devicepath, *dname;

		if((u_long)NULL==addr) break;

                if(-1==kvm(addr, esp[0])){
			Error("kvm read: esp #%d", j);
		}

		/* 
		 * grab the devinfo structure for the esp controller. CNUM 
		 * requires it, and we need it to determine the device names.
		 */
		if(-1==kvm(esp->e_dev, devi)){
			Error("kvm read: #%d esp devinfo", j);
		} 

		devicepath = get_devicepath(j);

		esp->e_dev=(dev_info_t)&devi; /* for CNUM */

		if(param_is_set(P_CONTROLLER)) {
			esp_controller_message(CNUM, esp);
		}

		init_scsitarget();

		/* 
		 * Now, traverse esp's devinfo child list, looking for
		 * attached drivers (i.e. DDI_CF2 is true). If we find
		 * an attached driver, we presume it is an active device.
		 */

		for(next=ddi_get_child(&devi);
		    NULL!=next; 
		    next=ddi_get_next_sibling(&di)){
			char *devicename, *addrname;
			int target, lun;

			if(-1==kvm(next, di)){
			    Error("kvm read: #%d esp devinfo child", j);
			}

			if(!DDI_CF2(&di)) continue; /* no device attached */

			devicecount++;

			/*
			 * Get the device name and address, and 
			 * compute the device path for it
			 */

			/* get device name */
			dname=s(kvmstr((u_long)ddi_get_name(&di)));
			/* ... and address */
			addrname=s(get_addr(&di));

#ifdef CENTRAL_DATA_CT
			if(STREQ(CENTRAL_DATA_CT, dname)) {
				/* 
				 * The Central Data STS driver attach
				 * routine always returns success, so 
				 * the kernel thinks that there's an STS
				 * at every target. But the only targets
				 * that really have an STS are the ones
				 * that have a non-zero devi_minor. 
				 */
				if(!di.devi_minor) continue;
			}
#endif /* CENTRAL_DATA_CT */

			devicename = compute_devicename(dname, 
						devicepath, addrname);

			if(-1==get_scsiaddr(dname, &di, &target, &lun)){
				Error("scsiaddr: #%d", j);
			}

			if(0==lun && (esp->e_weak & (1<<target))) noisycount++;

			banner(param_is_set(P_FORMATDAT),
				param_is_set(P_REPORT), devicecount);
			if(!param_is_set(P_QUIET)){
				if(param_is_set(P_SUNOS4STYLE)) {
					if(lun==0){
						boot_synch_message("esp",
							CNUM, target, 
							esp_get_speed(esp, 
								target), 
								NARROW);
					}
				} else {
					printf("esp%d: %s%s tgt %d lun %d:\n", 
						CNUM, dname, addrname, 
						target, lun);
					if(param_is_set(P_SYNCH)){
						printf("\t");
						esp_synch_message(esp, target, 
						 esp_get_speed(esp, target));
						printf("\n");
					}
				}
			} 
			if(param_is_set(P_PROBE)) {
				do_scsi_probe(dname, devicename, lun);
			}
			if(param_is_set(P_REPORT)){
				report(param_is_set(P_FORMATDAT), 
					osrelease, devicename,
					param_is_set(P_ORIGINAL));
			}
			free(devicename);
			free(addrname);
			free(dname);
		}

		free(devicepath);
#ifdef OLD_SUNOS
		/* SunOS 4.1.1 and previous use a table of esp structures */
		addr=(u_long)(++esp_p);
#else
		/* SunOS 4.1.2 and later use a linked list of esp structures */
                addr=(u_long)esp->e_next;
#endif

	}
	return(noisycount);
}

#endif /* ESP_SUPPORT */
