/*
 * isp.c  -  process ISP host adapters.
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         <jdd@cdf.toronto.edu>
 * 
 * ISP support by: Calum Mackay, Sun Microsystems
 *                 <calum.mackay@uk.sun.com>
 */

#ifndef lint
static char rcsid[] = "$Id: isp.c,v 1.13 1999/04/22 17:40:16 jdd Exp $";
#endif

#ifdef ISP_SUPPORT

#include <stdio.h>
#ifdef FILENAME_MAX	/* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#ifdef SYSV
#include "sunos5.h"

#ifdef SCSI_HBA_TRAN_CLONE /* SunOS 5.4 or later */
#include <sys/scsi/adapters/ispmail.h>
#include <sys/scsi/adapters/ispvar.h>
/* 
 * It's ugly and dangerous to redefine CNUM here, but SunOS 5.4 ispvar.h's 
 * idea of CNUM is completely wrong. It uses data structures that existed in
 * 2.3 but no longer exist in 2.4.
 */
#undef CNUM
#define CNUM(isp)               (ddi_get_instance(isp->isp_tran->tran_hba_dip))
#ifndef ISP_20M_SYNC_PERIOD
#define ISP_20M_SYNC_PERIOD     0x000C
#endif /* ISP_20M_SYNC_PERIOD */
#else /* !SCSI_HBA_TRAN_CLONE */
#include "ispvar.h"
#endif /* !SCSI_HBA_TRAN_CLONE */

#include "uscsi.h"
#include "format.h"

#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <nlist.h>
#include <kvm.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include "utils.h"
#include "param.h"
#include "common.h"

typedef struct {
        int     period;         /* ISP_xM_SYNC_PERIOD */
        int     speed;          /* x000 */
} isp_speed_mapping;

static isp_speed_mapping isp_speed[] = {
        ISP_20M_SYNC_PERIOD, 20000,
        ISP_10M_SYNC_PERIOD, 10000,
        ISP_8M_SYNC_PERIOD,   8000,
        ISP_5M_SYNC_PERIOD,   5000,
        ISP_4M_SYNC_PERIOD,   4000
#define ISP_PERIOD_TABLE 5      /* Number of elements in isp_speed[] */
};

static u_int isp_get_speed(isp, i)
struct isp *isp;
int i; /* target index */
{
	int index;

	for ( index = 0; index < ISP_PERIOD_TABLE; index++ )
		if ( isp_speed[index].period == PERIOD_MASK(isp->isp_synch[i]) )
			return(isp_speed[index].speed);

	return((u_int)-1);
}

static void isp_synch_message(isp, i, speed)
struct isp *isp;
int i;
u_int speed;
{
	int width = NARROW;
	if(isp->isp_cap[i] & ISP_CAP_WIDE) width = WIDE;

	switch (speed) {
		case (u_int)-1 :
			if ( isp->isp_cap[i] & ISP_CAP_SYNC ) {
				printf("Synchronous(Unknown MB/sec)" );
			}
			else {
				printf("Asynchronous");
			}
			break;
		case 0 :
			printf("Asynchronous");
			break;
		default :
			if(speed && (isp->isp_cap[i] & ISP_CAP_SYNC)){
				printf("Synchronous(%d.%.3dMB/sec)",
					(width/8)*speed/1000, 
					(width/8)*speed%1000);
			}
			else {
				printf("Asynchronous"); 
			}
			break;
	}
	printf(" Unknown"); /* isp driver doesn't keep track of errors */
	if(isp->isp_cap[i] & ISP_CAP_DISCONNECT){
		printf(" CanReconnect");
	} else {
		printf(" CannotReconnect");
	}
	if(isp->isp_cap[i] & ISP_CAP_TAG){
		printf(" TaggedQueuing");
	} else {
		printf(" NoTaggedQueuing");
	}
	if(width == WIDE){
		printf(" Wide");
	} else {
		printf(" Narrow");
	}
}

static void isp_controller_message(cnum, isp)
int cnum;
struct isp *isp;
{
	printf("isp%d:", cnum);
	printf(" QLGC ISP 1000" );
	printf(" firmware revision %d.%d\n", (int) isp->isp_major_rev,
						(int) isp->isp_minor_rev );
}

int process_isp()
{
	u_long addr;
	struct isp isp[1];
	u_int j, nisp;
	int noisycount=0;

	if(-1==kvm(nl[N_ISP_HEAD].n_value, addr)){
		return(0); /* isp driver not loaded */
	}

	if((u_long)NULL==addr) return(0); /* no isp's */

        if(-1==kvm(addr, isp)){
		Error("kvm read: initial isp");
	}

	nisp=get_nisp();
	if(0==nisp) return(0);

	for(j=0;j<nisp;j++){
		struct dev_info di, *next;
		char *devicepath, *dname;

		if((u_long)NULL==addr) break;

                if(-1==kvm(addr, isp[0])){
			Error("kvm read: isp #%d", j);
		}

#ifdef SCSI_HBA_TRAN_CLONE
		{
			scsi_hba_tran_t tran;
			/* 
			 * Grab the transport devinfo structure for the isp 
			 * controller. Needed for CNUM. 
			 */
			if(-1==kvm((u_long)isp->isp_tran, tran)){
				Error("kvm read: #%d isp tran", j);
			}
			isp->isp_tran=(scsi_hba_tran_t *)&tran; 
			if(-1==kvm((u_long)isp->isp_tran->tran_hba_dip, devi)){
				Error("kvm read: #%d isp devinfo", j);
			}
			isp->isp_tran->tran_hba_dip=(dev_info_t *)&devi;
		}
#else /* !SCSI_HBA_TRAN_CLONE */
		/* 
		 * grab the devinfo structure for the isp controller. CNUM 
		 * requires it, and we need it to determine the device names.
		 */
		if(-1==kvm((u_long)isp->isp_dip, devi)){
			Error("kvm read: #%d isp devinfo", j);
		}
		isp->isp_dip=(dev_info_t)&devi; /* for CNUM */
#endif /* !SCSI_HBA_TRAN_CLONE */

		devicepath = get_devicepath(j);

		if(param_is_set(P_CONTROLLER)) {
			isp_controller_message(CNUM(isp), isp);
		}

		init_scsitarget();

		/* 
		 * Now, traverse isp's devinfo child list, looking for
		 * attached drivers (i.e. DDI_CF2 is true). If we find
		 * an attached driver, we presume it is an active device.
		 */

		for(next=ddi_get_child(&devi);
		    NULL!=next; 
		    next=ddi_get_next_sibling(&di)){
			char *devicename, *addrname;
			int target, lun;

			if(-1==kvm((u_long)next, di)){
			    Error("kvm read: #%d isp devinfo child", j);
			}

			if(!DDI_CF2(&di)) continue; /* no device attached */

			devicecount++;

			/*
			 * Get the device name and address, and 
			 * compute the device path for it
			 */

			/* get device name */
			dname=s(kvmstr((u_long)ddi_get_name(&di)));
			/* ... and address */
			addrname=s(get_addr(&di));

#ifdef CENTRAL_DATA_CT
			if(STREQ(CENTRAL_DATA_CT, dname)) {
				/* special case for Central Data STS driver */
				if(!di.devi_minor) continue;
			} 
#endif /* CENTRAL_DATA_CT */

			devicename = compute_devicename(dname, 
						devicepath, addrname);

			if(-1==get_scsiaddr(dname, &di, &target, &lun)){
				Error("scsiaddr: #%d", j);
			}

			banner(param_is_set(P_FORMATDAT), 
				param_is_set(P_REPORT), devicecount);
			if(!param_is_set(P_QUIET)){
				if(param_is_set(P_SUNOS4STYLE)) {
					if(lun==0){
						boot_synch_message("isp",
							CNUM(isp), target, 
						  	isp_get_speed(isp,
								target),
								WIDE);
					}
				} else {
					printf("isp%d: %s%s tgt %d lun %d:\n", 
						CNUM(isp), dname, addrname, 
						target, lun);
					if(param_is_set(P_SYNCH)){
						printf("\t");
						isp_synch_message(isp, target, 
						 isp_get_speed(isp, target));
						printf("\n");
					}
				}
			}

			if(param_is_set(P_REPORT)){
				report(param_is_set(P_FORMATDAT), 
					osrelease, devicename,
					param_is_set(P_ORIGINAL));
			}
			if(param_is_set(P_PROBE)){
				do_scsi_probe(dname, devicename, lun);
			}
			free(devicename);
			free(addrname);
			free(dname);
		}
		free(devicepath);
                addr=(u_long)isp->isp_next;
	}

	return(noisycount);

}
#else /* !SYSV */
int process_isp(){}
#endif /* !SYSV */
#endif /* ISP_SUPPORT */
