/*
 * sunos4.c: SunOS 4.1.x dependent code. 
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         jdd@cdf.toronto.edu
 *
 * with modifications by Michael Lowry, Princeton University, Physics
 *         mml@pupcyc2.princeton.edu
 *    to use the scsi_device list to obtain target and lun values
 * 
 * $Id: sunos4.c,v 1.4 1997/04/20 21:46:39 jdd Exp $
 */

#include <stdio.h>
#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#ifndef SYSV
#include <nlist.h>
#include <scsi/impl/types.h>
#include "sunos4.h"
#include "common.h"
#include "utils.h"

struct nlist nl[]={
        { "_nesp" },
        { "_esp_softc" },
        { "_sd_root" },
	{ "_nptisp" },
	{ "_ptisp_softc" },
        { "" },
};

struct dev_info devi; /* device info from kernel, used in OS-dependent files */

static struct target_lookup {
	struct scsi_conf *sc;
	struct target_lookup *next;
} *sctable = NUL(struct target_lookup *);

#ifdef ESP_SUPPORT
/* Get number of ESP controllers */
u_int get_nesp(){
	u_int  nesp;
        if(-1==kvm(nl[N_NESP].n_value, nesp)) {
                Error("kvm_read: nesp");
        }
	return(nesp);
}
#endif /* ESP_SUPPORT */

#ifdef ISP_SUPPORT
/* Get number of ISP controllers */
u_int get_nisp(){
	return(0); /* ISP controllers not supported under SunOS 4.x */
}
#endif /* ISP_SUPPORT */

#ifdef FAS_SUPPORT
/* Get number of FAS controllers */
u_int get_nfas(){
	return(0); /* FAS controllers not supported under SunOS 4.x */
}
#endif /* FAS_SUPPORT */

#ifdef PTISP_SUPPORT
/* Get number of PTISP controllers */
u_int get_nptisp(){
	u_int  nptisp;
	if (N_UNDF == nl[N_NPTISP].n_type) {
		return((u_int)0);
	} else if(0>kvm(nl[N_NPTISP].n_value, nptisp)) {
		Error("kvm_read: nptisp");
	}
	return(nptisp);
}
#endif /* PTISP_SUPPORT */

/* Get path to specified controller. Returned string is malloc'ed */
char *get_devicepath(j)
int j;
{
	return(s("/dev"));
}

/* Get address of specified device. Note: returned in static storage */
char *get_addr(di)
struct dev_info *di;
{
	static char buff[20];
	sprintf(buff, "%d", ddi_get_unit(di));
	return(buff);
}

void init_scsitarget(){
	/* Grab scsi_conf table */
/* Actually construct a psuedo one from the scsi_device linked list */

	struct scsi_conf *sctmp; 
	struct target_lookup *tltmp;
	struct scsi_device sdtmp;
	struct dev_info ditmp;
	u_long tableaddr, sd_ptr;

	tableaddr=(u_long)nl[N_SD_ROOT].n_value;
	if(-1==kvm(tableaddr, sd_ptr)){
		Error("get kvm: sd_root\n");
	}
	while (sd_ptr) {
		sctmp=new(struct scsi_conf);
		/* get trgt & lun from scsi_device structure */
		if(-1==kvm(sd_ptr, sdtmp)){
			Error("get kvm: scsi_device\n");
		}
                sctmp->target= (char)sdtmp.sd_address.a_target;
                sctmp->lun   = (char)sdtmp.sd_address.a_lun;

		/* get unit & name from associated dev_info structure */
		if(-1==kvm(sdtmp.sd_dev, ditmp)){
			Error("get kvm: scsi_dev_info\n");
		}
		sctmp->dunit = (char)ditmp.devi_unit;
		sctmp->tname = s(kvmstr(ditmp.devi_name));

		/* add new entry to lookup table */
		tltmp = new(struct target_lookup);
		tltmp->next = sctable;
		tltmp->sc = sctmp;
		sctable=tltmp;

		sd_ptr  = (u_long)sdtmp.sd_next;
	}
}

int get_scsiaddr(name, di, target, lun)
char *name;
struct dev_info *di;
int *target, *lun;
{
	int unit;
	struct target_lookup *i;

	unit=ddi_get_unit(di);
	for(i=sctable; NULL!=i; i=i->next){
		if((unit==(int)i->sc->dunit) &&
		   STREQ(name, i->sc->tname)){
			*target = (int)i->sc->target;
			*lun = (int)i->sc->lun;
			return(0);
		}
	}
	return(-1);
}
#endif /* !SYSV */
