/*
 * sunos5.c: SunOS 5.x dependent code. 
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         jdd@cdf.toronto.edu
 *
 * $Id: sunos5.c,v 1.9 1999/04/22 17:38:49 jdd Exp $
 */

#include <stdio.h>
#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#ifdef SYSV
#include "sunos5.h"
#include "common.h"
#include "utils.h"

struct nlist nl[]={
        { "esp_softc" },
	{ "isp_head" },
	{ "fas_head" },
	{ "ptisp_head" },
	{ "glm_head" },
        { "" },
};

struct dev_info devi; /* device info from kernel, used in OS-dependent files */

#ifdef ESP_SUPPORT
/* Get number of ESP controllers */
u_int get_nesp(){
        return((u_int)255);
}
#endif /* ESP_SUPPORT */

#ifdef ISP_SUPPORT 
/* Get number of ISP controllers */
u_int get_nisp(){
        return((u_int)255);
}
#endif /* ISP_SUPPORT */

#ifdef FAS_SUPPORT
/* Get number of FAS controllers */
u_int get_nfas(){
        return((u_int)255);
}
#endif /* FAS_SUPPORT */

#ifdef PTISP_SUPPORT
/* Get number of PTISP controllers */
u_int get_nptisp(){
	if (!nl[N_PTISP_HEAD].n_type){
		return((u_int)0);
	} else {
		return((u_int)255);
	}
}
#endif /* PTISP_SUPPORT */

#ifdef GLM_SUPPORT
/* Get number of GLM controllers */
u_int get_nglm(){
        return((u_int)255);
}
#endif /* GLM_SUPPORT */

/* 
 * Get path to specified controller device. Returned string is malloc'ed.
 * assumption: global variable devi points to controller device info.
 */
char *get_devicepath(j)
unsigned int j;
{
	char *devicepath, *cp;

	/* Grab the name */
	devicepath=s(kvmstr((u_long)ddi_get_name(&devi)));
	/* ... and the address */
	cp=cat(devicepath, "@", kvmstr((u_long)ddi_get_addr(&devi)), NULL);
	free(devicepath);
	devicepath=cp;
	/*
	 * At this point, devicepath should be eg. esp@addr,addr. Now
	 * trace backwards up the devinfo tree until we get to the
	 * root. At each branch, record the name and address in
	 * devicepath as in "...device@addr,addr/device@addr,addr/..."
	 */
	{
		/* construct path */
		struct dev_info di, *parent;
		char *name;

		for(parent=ddi_get_parent(&devi);
		    NULL!=parent;
		    parent=ddi_get_parent(&di)){
			/* grab the parent */
			if(-1==kvm((u_long)parent, di)){
				Error("kvm read: #%d parent %s", j,
					devicepath);
			}
			/* get its name */
			name=kvmstr((u_long)ddi_get_name(&di));

			/* 
			 * If this name is "pci[0-9]*", there's a PCI address
			 * in the name that we want to get rid of.  
			 */
#define 		PCI "pci"
			if(0==strncmp(PCI, name, strlen(PCI)) &&
					isdigit(name[strlen(PCI)])){
				name[strlen(PCI)]='\0'; 
			}

			/* get the address, add both to devicepath */
			if(DDI_CF1(&di)){
				char *pathcomp, *newpath;

				pathcomp=cat(name, "@", NULL);
				name=kvmstr((u_long)ddi_get_addr(&di));
				if(!*name){
					free(pathcomp);
				} else {
					newpath = cat(pathcomp,
							name,
							"/",
							devicepath,
							NULL);
					free(devicepath);
					free(pathcomp);
					devicepath=newpath;
				}
			}
		}
	}
	/* almost done. just add "/devices/" to head of devicepath */
	cp=cat("/devices/", devicepath, NULL);
	free(devicepath);
	devicepath=cp;

	if(0>access(devicepath, F_OK) && errno==ENOENT){
		/* 
		 * Devicepath doesn't work.  We need to apply a kludge
		 * or two.  The PCI GLM driver replaces "glm" (for U5/10)
		 * or the 2nd occurrence of "pci" (for U250/U450) with "scsi";
		 * Rather than try to keep track of the (inconsistent)
		 * differences between various Ultras, we'll just stroll along
		 * devicepath until we find out where it stops working, and
		 * replace the problematic component with "scsi".
		 */
		char *newpath = cat("/", strtok(devicepath, "/"), NULL);
		while(NULL!=(cp=strtok(NULL, "/"))){
			char *testpath = cat(newpath, "/", cp, NULL);
			if(0>access(testpath, F_OK) && errno==ENOENT){
				/* 
				 * Testpath doesn't work -- fix this 
				 * component.  
				 */
				char *f = newpath;
				newpath=cat(newpath, "/scsi", strchr(cp, '@'), 
					NULL);
				free(f);
				free(testpath);
			} else {
				/* component is ok; testpath is good */
				free(newpath);
				newpath=testpath;
			}
		}
		{ char *f = devicepath; devicepath = newpath; free(f); }
	}
	return(devicepath);
}

/* Get address of specified device. Note: returned in static storage */
char *get_addr(di)
struct dev_info *di;
{
	return(kvmstr((u_long)ddi_get_addr(di)));
}

void init_scsitarget(){}

int get_scsiaddr(name, di, target, lun)
char *name;
struct dev_info *di;
int *target, *lun;
{
	struct scsi_device sd;

	if(-1==kvm((u_long)ddi_get_driver_private(di), sd)){
		return(-1);
	}
	*target = (int)sd.sd_address.a_target;
	*lun = (int)sd.sd_address.a_lun;
	return(0);
}
#endif /* SYSV */
