/*
 * sunos5.h: SunOS 5.x dependent code. 
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         jdd@cdf.toronto.edu
 *
 * $Id: sunos5.h,v 1.20 1998/02/28 02:31:40 jdd Exp $
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/map.h>
#include <signal.h>

/* 
 * The following includes are pretty messy.
 * 
 * We want some of the kernel-level definitions in sys/ddi.h and sys/sunddi.h
 * but they're not accessible in 2.5beta unless _KERNEL is defined.  And
 * cmn_err.h is included by sunddi.h, but if it's included with _KERNEL
 * defined, it defines prototypes for various functions that are incompatible
 * with the prototypes in stdio.h and stdlib.h.  So include it first, without
 * _KERNEL, so it'll be skipped later.  Also, there's a prototype for swab in
 * sunddi.h in 2.5beta that's incompatible with the one in stdlib, so we
 * redefine swab to something else.  Ditto for the str* routines in 2.1.
 */ 
#include <sys/cmn_err.h>
#ifndef _KERNEL
#define _KERNEL
#endif /* !_KERNEL */
/* 
 * In solaris 2.6, with some versions of gcc, we need _LARGEFILE64_SOURCE
 * defined here, else there'll be parse errors in sys/resource.h (which is 
 * included  from sys/vnode.h, which is included from sys/stream.h, which is 
 * included from sys/ddi.h)
 */
#ifdef _gcc_
SYNTAX ERROR
#ifndef _LARGEFILE64_SOURCE
#define _LARGEFILE64_SOURCE
#endif /* _LARGEFILE64_SOURCE */
#endif /* _gcc_ */
#include <sys/ddi.h>
#define strcpy _badstrcpy
#define strncpy _badstrncpy
#define strchr _badstrchr
#define strcmp _badstrcmp
#define strncmp _badstrncmp
#define swab _badswab  
#include <sys/sunddi.h>
#undef strcpy
#undef strncpy
#undef strchr
#undef strcmp
#undef strncmp
#undef swab
#undef _KERNEL
#include <sys/ddi_impldefs.h>
#include <sys/scsi/conf/device.h>
#define _KERNEL
#include <sys/scsi/impl/transport.h>
#undef  _KERNEL

#include "common.h"

extern struct dev_info devi;

/* make sure these match up with the def'n of nl in sunos5.c */
#define N_ESP_SOFTC 0
#define N_ISP_HEAD 1
#define N_FAS_HEAD 2
#define N_PTISP_HEAD 3
#define N_GLM_HEAD 4
extern struct nlist nl[];

#define ddi_get_instance(a) ((struct dev_info *)a)->devi_instance
#define ddi_get_name(a) ((struct dev_info *)a)->devi_name
#define ddi_get_addr(a) ((struct dev_info *)a)->devi_addr
#define ddi_get_child(a) ((struct dev_info *)a)->devi_child
#define ddi_get_parent(a) ((struct dev_info *)a)->devi_parent
#define ddi_get_next_sibling(a) ((struct dev_info *)a)->devi_sibling
#define ddi_get_driver_private(a) ((struct dev_info *)a)->devi_driver_data

/*
 * device name is DEV_PREFIX<device>DEV_INFIX<addr>DEV_SUFFIX,
 * disk device name is DEV_PREFIX<device>DEV_INFIX<addr>DISK_SUFFIX,
 * 
 * For SunOS5: <device>@<addr>: or <device>@<addr>:c,raw for disk
 */
#define DEV_PREFIX ""
#define DEV_INFIX "@"
#define DEV_SUFFIX ":"
#define DISK_SUFFIX ":c,raw"
#define TAPE_PREFIX ""
#define TAPE_SUFFIX ":lbn"

extern void init_scsitarget();
extern int get_scsiaddr();
extern char *get_addr();
extern char *get_devicepath();
#ifdef ESP_SUPPORT
extern u_int get_nesp();
#endif /* ESP_SUPPORT */
#ifdef ISP_SUPPORT
extern u_int get_nisp();
#endif /* ISP_SUPPORT */
#ifdef FAS_SUPPORT
extern u_int get_nfas();
#endif /* FAS_SUPPORT */
#ifdef PTISP_SUPPORT
extern u_int get_nptisp();
#endif /* PTISP_SUPPORT */
#ifdef GLM_SUPPORT
extern u_int get_nglm();
#endif /* GLM_SUPPORT */
