/*
 * uscsi.c - User SCSI routines for querying devices.
 *
 * Author: John DiMarco,
 *         University of Toronto, Computing Disciplines Facility
 *         jdd@cdf.toronto.edu
 */

#ifndef lint
static char rcsid[] = "$Id: uscsi.c,v 3.25 1998/08/07 20:56:53 jdd Exp $";
#endif

#include <stdio.h>
#ifdef FILENAME_MAX     /* defined in stdio.h only in SYSV systems */
#define SYSV
#include <string.h>
#else
#include <strings.h>
#endif

#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include "utils.h"
#include "uscsi.h"

extern int d;

#ifdef SYSV

#include <stdlib.h>

/* silly workaround for a bug in gcc <<2.6.2 for Solaris 2.4 */
#include <sys/byteorder.h>
#if !defined(_BIT_FIELDS_LTOH) && !defined(_BIT_FIELDS_HTOL)
#ifdef __i386__
#define _BIT_FIELDS_LTOH
#else /* !__i386__ */
#define _BIT_FIELDS_HTOL
#endif /* !__i386__ */
#endif /* !defined(_BIT... */
#ifdef _BIT_FIELDS_HTOL
#ifndef ntohl
#define ntohl(x) (x)
#define ntohs(x) (x)
#endif /* ndef ntohl */
#endif /* _BIT_FIELDS_HTOL */

#include <sys/scsi/generic/mode.h>
#include <sys/scsi/generic/commands.h>
#include <sys/scsi/impl/types.h>
#include <sys/scsi/impl/uscsi.h>
#include <sys/systeminfo.h>
#define gethostname(b, l) sysinfo(SI_HOSTNAME, b, (long)l)
#define bzero(b,l) memset(b,0,l)

#else /* SYSV */

#include <scsi/impl/types.h>
#include <scsi/impl/uscsi.h>

#endif /* SYSV */

#define SCSIBUFLEN 8000

int scsi_read_blklim(fd, max_bll, min_bll)
int fd;
unsigned int *max_bll, *min_bll; 
{
        struct uscsi_cmd cmd;
        union scsi_cdb cdb;
        struct scsi_read_blklim {
                unsigned char reserved;
                unsigned char max[3];
                unsigned char min[2];
        } *rbl;
        unsigned char buff[SCSIBUFLEN];

        rbl = (struct scsi_read_blklim *)buff;

        /* set up SCSI read_blklim command */
        bzero(&cdb, sizeof(cdb));
        bzero(rbl, sizeof(rbl));
        cdb.scc_cmd = SCMD_READ_BLKLIM;
        cmd.uscsi_cdb=(caddr_t)&cdb;
        cmd.uscsi_cdblen=6;  /* SCSI Group 0 cmd */
        cmd.uscsi_bufaddr=(caddr_t)rbl;
        cmd.uscsi_buflen=SCSIBUFLEN;
        cmd.uscsi_flags=USCSI_READ;

        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);

/* {int i; for(i=0;i<6;i++) printf("%.2X ", (u_int)buff[i]); printf("\n");} */

	*max_bll=rbl->max[0]*(256*256)+rbl->max[1]*256+rbl->max[2];
	*min_bll=rbl->min[0]*256+rbl->min[1];

        return(0);
}

char *scsi_dtype(dtype)
u_int dtype;
{
	switch(dtype){
	case DTYPE_DIRECT: 	return("Disk");
	case DTYPE_SEQUENTIAL: 	return("Tape");
	case DTYPE_PRINTER: 	return("Printer");
	case DTYPE_PROCESSOR: 	return("Processor");
	case DTYPE_WORM:	return("WORM");
	case DTYPE_RODIRECT:	return("CD-ROM");
	case DTYPE_SCANNER:	return("Scanner");
	case DTYPE_OPTICAL:	return("Optical");
	case DTYPE_CHANGER:	return("Changer");
	case DTYPE_COMM:	return("Comm");
	default:		return("Unknown");
	}
}

int scsi_probe(devname, lun)
char *devname;
int lun;
{
	int dfd;
	struct uscsi_cmd cmd;
	union scsi_cdb cdb;
	struct scsi_inquiry *inq;
	char buff[SCSIBUFLEN]; 

	if(-1==(dfd=open(devname, O_RDONLY|O_NDELAY))) return(-1);

	inq = (struct scsi_inquiry *)buff;
	
	/* set up SCSI inquiry command */
	bzero(&cdb, sizeof(cdb));
	bzero(inq, sizeof(inq));
	cdb.scc_cmd = SCMD_INQUIRY;
	cdb.scc_lun = lun;
 	cdb.g0_count0 = 200;
	cmd.uscsi_cdb=(caddr_t)&cdb;
	cmd.uscsi_cdblen=6; /* SCSI Group 0 cmd */
	cmd.uscsi_bufaddr=(caddr_t)inq;
	cmd.uscsi_buflen=SCSIBUFLEN;
	cmd.uscsi_flags=USCSI_DIAGNOSE|USCSI_SILENT|USCSI_ISOLATE|USCSI_READ;

	if(-1==ioctl(dfd, USCSICMD, &cmd)) { (void)close(dfd); return(-1); }
	(void)close(dfd);

	printf("%s ", inq->inq_rmb?"Removable":"Non-removable");
	printf("%s:\t", scsi_dtype(inq->inq_dtype));
	printf("%-8.8s %-16.16s %-4.4s\t\t[%s%s%s%s%s]", inq->inq_vid, 
		inq->inq_pid, inq->inq_revision, 
		(inq->inq_reladdr?"A":""), (inq->inq_sync?"S":""),
		(inq->inq_linked?"L":""), (inq->inq_cmdque?"T":""),
		(inq->inq_sftre?"R":""));
	return(0);
}

int scsi_inquiry(fd, devicetype, vendor, product, revision, serial, capability)
int fd, *devicetype, *capability;
char *vendor, *product, *revision, *serial;
{
        struct uscsi_cmd cmd;
        union scsi_cdb cdb;
        struct scsi_inquiry *inq;
        char buff[SCSIBUFLEN];

        inq = (struct scsi_inquiry *)buff;

        /* set up SCSI inquiry command */
        bzero(&cdb, sizeof(cdb));
        bzero(inq, sizeof(inq));
        cdb.scc_cmd = SCMD_INQUIRY;
        cdb.g0_count0 = 200;
        cmd.uscsi_cdb=(caddr_t)&cdb;
        cmd.uscsi_cdblen=6; /* SCSI Group 0 cmd */
        cmd.uscsi_bufaddr=(caddr_t)inq;
        cmd.uscsi_buflen=SCSIBUFLEN;
        cmd.uscsi_flags=USCSI_DIAGNOSE|USCSI_SILENT|USCSI_ISOLATE|USCSI_READ;

        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);

	*devicetype=inq->inq_dtype;
	strncpy(vendor, inq->inq_vid, sizeof(inq->inq_vid));
	strncpy(product, inq->inq_pid, sizeof(inq->inq_pid));
	strncpy(revision, inq->inq_revision, sizeof(inq->inq_revision));
	strncpy(serial, inq->inq_revision+REVISION_SIZE, SERIAL_SIZE);
	/* Could use inq_serial in the previous line, but it's only defined 
	 * in the SunOS 5.x headers, unfortunately, and even there, it's the
	 * wrong size (12 bytes, not 20 as the SCSI2 spec dictates) so I'll 
	 * hard-code its size and location instead. */

	/* 
	 * set capability -- it contains byte 7 (all the capability bit 
	 * flags) set the way we want them.  We can't just pass the byte 
	 * "as is", since the bits are in a different order depending on
	 * endien-ness.
	 */ 
	if(inq->inq_reladdr) *capability |= INQ_RELADR;
	if(inq->inq_wbus32)  *capability |= INQ_WBUS32;
	if(inq->inq_wbus16)  *capability |= INQ_WBUS16;
	if(inq->inq_sync)    *capability |= INQ_SYNC;
	if(inq->inq_linked)  *capability |= INQ_LINKED;
	if(inq->inq_cmdque)  *capability |= INQ_CMDQUE;
	if(inq->inq_sftre)   *capability |= INQ_SFTRE;

	return(0);
}

int scsi_capacity(fd, capacity)
int fd; 
unsigned long *capacity;
{
        struct uscsi_cmd cmd;
        union scsi_cdb cdb;
        struct scsi_read_capacity {
		int sectors;
		int sector_size;
		char junk[2];
	} *inq;
        char buff[SCSIBUFLEN];

        inq = (struct scsi_read_capacity *)buff;

        /* set up SCSI read_capacity command */
        bzero(&cdb, sizeof(cdb));
        bzero(inq, sizeof(inq));
        cdb.scc_cmd = SCMD_READ_CAPACITY;
        cmd.uscsi_cdb=(caddr_t)&cdb;
        cmd.uscsi_cdblen=10;  /* SCSI Group 1 cmd */
        cmd.uscsi_bufaddr=(caddr_t)inq;
        cmd.uscsi_buflen=SCSIBUFLEN;
        cmd.uscsi_flags=USCSI_READ;

        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);

	*capacity=1+ntohl((u_long)inq->sectors);
	return(0);
}

/* 
 * Note carefully: 
 * scsi_modeinfo assumes the open device is a direct access device, and it
 * doesn't bother checking before going ahead.
 * 
 * Also, scsi_modeinfo won't set rpm if the drive doesn't return the
 * information (eg. SCSI-1 drives). 
 */

int scsi_modeinfo(fd, original, nsect, sector_size, pcyl, nhead, rpm, 
			trks_zone, asect, atrks, cache_status)
int fd, original;
int *nsect, *sector_size, *pcyl, *nhead, *rpm, *trks_zone, *asect, *atrks,
*cache_status;
{
        struct uscsi_cmd cmd;
        union scsi_cdb cdb;
        struct mode_format mfmt;
	struct mode_geometry mgeom;
	struct mode_cache mcache;
        char buff[SCSIBUFLEN];
	char *buffp;

        bzero(&cdb, sizeof(cdb));
        bzero(&mfmt, sizeof(mfmt));
	cdb.scc_cmd = SCMD_MODE_SENSE;
	/* cdb.g0_addr1 = MODEPAGE_ALLPAGES | (0x80 * original); */
	cdb.g0_count0 = 255;
	cmd.uscsi_cdb=(caddr_t)&cdb;
	cmd.uscsi_cdblen=6; /* SCSI Group 0 cmd */
	cmd.uscsi_bufaddr=(caddr_t)buff;
	cmd.uscsi_buflen=SCSIBUFLEN;
	cmd.uscsi_flags=USCSI_DIAGNOSE|USCSI_ISOLATE|USCSI_READ;
	
	/* grab the format page */
	cdb.g0_addr1 = DAD_MODE_FORMAT | (0x80 * original);
        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);
	buffp = MODE_PAGE_ADDR((struct mode_header *)buff, char);
	memset((char *)&mfmt, 0, sizeof(mfmt));
	memcpy((char *)&mfmt, buffp, sizeof(struct mode_page));
	memcpy((char *)&mfmt, buffp, mfmt.mode_page.length);

	*trks_zone = (int)ntohs((u_short)mfmt.tracks_per_zone);
	*asect = (int)ntohs((u_short)mfmt.alt_sect_zone);
	*nsect = (int)ntohs((u_short)mfmt.sect_track);
	*atrks = (int)ntohs((u_short)mfmt.alt_tracks_vol);
	*sector_size = (int)ntohs((u_short)mfmt.data_bytes_sect);

	/* grab the geometry page */
	cdb.g0_addr1 = DAD_MODE_GEOMETRY | (0x80 * original);
        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);
	buffp = MODE_PAGE_ADDR((struct mode_header *)buff, char);
	memset((char *)&mgeom, 0, sizeof(mgeom));
	memcpy((char *)&mgeom, buffp, sizeof(struct mode_page));
	memcpy((char *)&mgeom, buffp, mgeom.mode_page.length);

	*pcyl = (int)(mgeom.cyl_ub<<16) + (int)(mgeom.cyl_mb<<8) + 
		(int)mgeom.cyl_lb;
	*nhead = (int)mgeom.heads;

	/* 
	 * SCSI-1 devices don't report rpm, which was tacked onto
	 * the end of the mode_geometry structure in SCSI-2, so only 
	 * set rpm if enough data was returned to ensure the
	 * geometry structure was filled at least that far.
	 */
	if(mgeom.mode_page.length > (((u_int)&mgeom.rpm)-((u_int)&mgeom))){
		*rpm = (int)ntohs((u_short)mgeom.rpm);
	}

	/* grab the cache page */
        cdb.g0_addr1 = DAD_MODE_CACHE | (0x80 * original);
        if(-1==ioctl(fd, USCSICMD, &cmd)) return(-1);
        buffp = MODE_PAGE_ADDR((struct mode_header *)buff, char);
	memset((char *)&mcache, 0, sizeof(mcache));
	memcpy((char *)&mcache, buffp, sizeof(struct mode_page));
	memcpy((char *)&mcache, buffp, mcache.mode_page.length);

	*cache_status=(mcache.wce<<1)+(1-mcache.rcd);

	return(0);
}
